### Instructions

- Use a **concise** yet **descriptive** title;
- Determine whether your issue is a **bug report**, a **feature request**, or a **documentation request**;
- Choose the corresponding template block below and fill it in, replacing or deleting text in italics (surrounded by `_`) as appropriate;
- Delete the other template blocks and this header.

----

## Bug Report

### Affected tool(s)
_Tool name(s), special parameters?_

### Affected version(s)
- [ ] Latest public release version [version?]
- [ ] Latest development/master branch as of [date of test?]

### Description 
_Describe the problem below. Provide **screenshots** , **stacktrace** , **logs** where appropriate._

#### Steps to reproduce
_Tell us how to reproduce this issue. If possible, include command lines that reproduce the problem and provide a minimal test case._

#### Expected behavior
_Tell us what should happen_

#### Actual behavior
_Tell us what happens instead_

----

## Feature request

### Tool(s) involved
_Tool name(s), special parameters?_

### Description
_Specify whether you want a modification of an existing behavior or addition of a new capability._
_Provide **examples**, **screenshots**, where appropriate._

----

## Documentation request

### Tool(s) involved
_Tool name(s), parameters?_

### Description 
_Describe what needs to be added or modified._

----